import * as fs from "node:fs";
import path from "node:path";

type Metadata = {
  title: string;
};

function parseFrontmatter(fileContent: string) {
  let frontmatterRegex = /---\s*([\s\S]*?)\s*---/;
  let match = frontmatterRegex.exec(fileContent);
  let frontMatterBlock = match![1];
  let content = fileContent.replace(frontmatterRegex, "").trim();
  let frontMatterLines = frontMatterBlock.trim().split("\n");
  let metadata: Partial<Metadata> = {};

  frontMatterLines.forEach(line => {
    let [key, ...valueArr] = line.split(": ");
    let value = valueArr.join(": ").trim();
    value = value.replace(/^['"](.*)['"]$/, "$1"); // Remove quotes
    metadata[key.trim() as keyof Metadata] = value;
  });

  return { metadata: metadata as Metadata, content };
}

const documentationDirectory = path.join(process.cwd(), "documentation");

/**
 * Gets the files for the documentation.
 */
function getDocumentationFiles() {
  return fs.readdirSync(documentationDirectory);
}

function getDocumentationFileContent(file: string) {
  return fs.readFileSync(path.join(documentationDirectory, file), "utf-8");
}

export function getDocumentation() {
  const files = getDocumentationFiles();

  return files.map(file => {
    const { metadata, content } = parseFrontmatter(getDocumentationFileContent(file));
    let slug = path.basename(file, path.extname(file));

    return {
      metadata,
      content,
      slug,
    };
  });
}
